import requests
import json
from .utils import parse_json_response, check_status_code
import math
import requests
import hashlib
import os


class APIClient:
    def __init__(self, api_key, user_id):
        self.api_key = api_key
        self.base_url = 'https://api.docta.ai/api-key-'
        self.user_id = user_id

    def get_balance(self):
        headers = {
            'apiKey': self.api_key,
            'userId': self.user_id
        }
        response = requests.get(f'https://api.docta.ai/user/me', headers=headers)
        check_status_code(response)
        balance = parse_json_response(response)['usd']
        return f'Your Balance is: ${balance}'

    def get(self, model, params=None):
        headers = {
            'apiKey': self.api_key,
            'userId': self.user_id
        }
        payload = {
            "message": params
        }
        response = requests.get(f'{self.base_url + model}', headers=headers, params=payload)
        check_status_code(response)
        return parse_json_response(response)

    def post(self, model, data=None):
        headers = {
            'apiKey': self.api_key,
            'userId': self.user_id
        }
        payload = {
            "message": json.dumps(data)
        }
        response = requests.post(f'{self.base_url + model}', headers=headers, json=payload)
        check_status_code(response)
        return parse_json_response(response)

    def preference_data_cleaning(self, file_type, instruction_keys, chosen_keys, rejected_keys, user_email, file_path):
        hermes_url = 'https://api.docta.ai'

        def get_md5(file_bytes):
            md5 = hashlib.md5()
            md5.update(file_bytes)
            return md5.hexdigest()

        def get_base_file_name(file_path):
            return os.path.basename(file_path)

        def get_parent_parent_base(file_path):
            return os.path.basename(os.path.dirname(os.path.dirname(file_path)))

        file_name = get_base_file_name(file_path)
        folder_name = get_parent_parent_base(file_path)

        try:
            file_info = os.stat(file_path)
        except OSError as e:
            return

        chunk_size = 1024 * 1024 * 10  # 10MB
        num_chunks = math.ceil(file_info.st_size / chunk_size)

        try:
            with open(file_path, 'rb') as fi:
                file_bytes = fi.read()
        except IOError as e:
            return

        file_md5 = get_md5(file_bytes)
        client = requests.Session()
        headers = {
            'apiKey': self.api_key,
            'userId': self.user_id
        }

        for i in range(int(num_chunks)):
            start = i * chunk_size
            end = min(start + chunk_size, file_info.st_size)
            chunk = file_bytes[start:end]
            chunk_md5 = get_md5(chunk)

            # 'folderType': 'text', 'fileType': 'text', dongze 说这个不重要, 我暂时记为 text
            fields = {
                'md5Value': file_md5,
                'originalFilename': file_name,
                'fileSize': str(len(chunk)),
                'folderType': 'text',
                'folderPath': '',
                'fileType': 'text',
                'chunk': str(i),
                'start': '0',
                'end': str(int(num_chunks) - 1),
                'chunks': str(int(num_chunks)),
                'chunkMd5': chunk_md5,
            }

            files = {
                'multipartFile': (file_name, chunk, 'application/octet-stream')
            }

            response = client.post(hermes_url + '/file/upload', headers=headers, data=fields, files=files)
            if response.status_code != 200:
                return parse_json_response(response)

        # 'folderType': 'text' 暂时的
        merge_params = {
            'md5Value': file_md5,
            'originalFilename': file_name,
            'folderType': 'text',
        }

        merge_response = client.post(hermes_url + '/file/merge', headers=headers, data=merge_params)
        if merge_response.status_code != 200:
            return parse_json_response(merge_response)

        # 后端接收到文件之后, 将文件上传到s3, 通知ML service, 进行扣费, 通知前端成功
        params = {
            'originalFilename': file_name,
            'fileType': file_type,
            'instructionKeys': instruction_keys,
            'chosenKeys': chosen_keys,
            'rejectedKeys': rejected_keys,
            'userEmail': user_email
        }

        response = client.post(hermes_url + '/file/preference_data_cleaning', headers=headers, data=params)
        if response.status_code != 200:
            return parse_json_response(response)

        return "success"
